#--------------------------------------------------------------------------------
sub event
#--------------------------------------------------------------------------------
# Generic event creator
# Usage:
#   &event( 
#           "D",           (Severity: C, W, I, U, S, P)
#           1,             (Host #)
#           "Event Text",  (Text)
#           "Shorttxt",    (Short Text)
#           "DBA001I",     (MsgID)
#           "/tmp/zzz"     (Optional - File to append to msg)
#           'NOTIX'        (Optional - don't create ticket) 
#           "$emailaddr"   (Optional - email address for this event only) 
#           "H"            (Optional - email address for this event only) 
#         )
#--------------------------------------------------------------------------------
# Changes:
#  bb 07Mar03: big change - moved insert to events hist to the end. this means
#   that dups will not be logged any more - db will not grow so fast.
#  bb 27Oct05: another big change - New server_wide_event logic 
#  bb 10Oct11: Localized e_msgmod and changed ID2/ID3 logic. 
#  bb 11Jan12: Logic for new Event_EMail_For_Sev: DBC.
#--------------------------------------------------------------------------------
{
  ($e_sev, $e_h, $e_msg, $e_short, $e_msgid, $e_file, $e_notix, $e_email, $e_pri_override) = @_;

  #---- 2012/08/31 - Try something to eliminate duplicate non-suppression
  $e_msg =~ s/\[//g; 
  $e_msg =~ s/\]//g; 

  &debug($e_h, "EVENT - e_msgid=$e_msgid e_msg=$e_msg e_short=$e_short"); 

  my $zleftwidth = 125; 

  #---- This is reac by eventNotify
  $ev_attachments = ""; 

  my $zurl = ""; 
  my $zurl2 = ""; 
  my $ztitle = ""; 

  my $z_host     = ""; 
  my $z_userlist = ""; 
  my $z_msg      = ""; 
  my $timenum    = -1; 
  my $cache_savemsg = ""; 
  my $event_cksum = -1;

  my $dbc_event_email = -1; 
  my $event_notify_text = ""; 
  my $e_long = ""; 
  my $e_long_rows  = 0; 
  my $e_long_bytes = 0; 
  my $sql = ""; 
  my $savemsg = ""; 
  my $mx = ""; 
  my $e_cat = ""; 

  #---- Hard-coded FLAGS
  my $event_cache_to_db     = 1; 
  my $suppress_dups_from_db = 1; 
  my $event_cache_to_file   = 1; 

  my $ticket_suppress_NTFE = 0; 

  #---- Copy $ev_cat from caller and NULL it 
  $e_cat  = $ev_cat;
  $ev_cat = ""; 

  #---- New 2015/05/12 - Notify that we're in event
  &logit("event - ($e_msgid) $e_msg",$e_h);

  #---- LOGIC Start
  if ($events_dbc[$e_h] eq "N")
  {
    &logit("Events: Set to N for this instance - NOT performing event notification",$e_h);
    &debug($e_h, "Events: Set to N for this instance"); 
    return;
  }

  #---- Variables
  $e_file_grep_exclude = "^Stack|^instanceDetails:"; 
  $e_msgmod = ""; 

  #---- Remove all quotes and shorten if neccessary

  $_ = $e_msg;
  s/'//g;
  s/"//g;
  s/\n/ /g;
  $e_msg = $_;
  $e_msg = substr($e_msg,0,490) if length($e_msg) > 490;
  $ticket_id = "N";

  $e_msgid = "${e_msgid}${e_sev}" if length($e_msgid) < 7;
  $e_msgid_short = substr($e_msgid,0,6);

  #---- Read file contents into $e_long for other uses
  if ($e_file ne "") 
  {
    #---- OLD
    ## $e_long = `cat $e_file | egrep -a -v '$e_file_grep_exclude'`;

    #---- NEW
    open(E_LONG,"$e_file") || &error_msg("Cannot open input $e_file - $!"); 
    while(<E_LONG>)
    {
      #---- Excludes
      next if /Stack/ || /instanceDetails:/; 
      $e_long .= $_; 
      $e_long_rows++; 
      $e_long_bytes += length($_); 
    }
    close(E_LONG); 
  }

  if ($server_wide_events)
  {
    #---- See if this event is a server-wide-event. if it is, then see
    #----   if this hostname-userid has reported this for this iteration
    #----   if this is true, then skip this event
    &logit("DEBUG - e_msgid_short=$e_msgid_short swe=$server_wide_events server_events_this_iter{$hostname_userid[$e_h]}=$server_events_this_iter{$hostname_userid[$e_h]}",$e_h);
    if ($server_wide_events =~ /$e_msgid_short/)
    {
      &logit("DEBUG - This event ($e_msgid_short) is specified a server-wide-event",$e_h);

      #---- Has this event already occured during this iteration for this server_userid combo?
      if ($server_events_this_iter{$hostname_userid[$e_h]} =~ /$e_msgid_short/)  
      {
	&logit("This event ($e_msgid_short) has already occured for server/userid $hostname_userid[$e_h] - suppressing",$e_h);
        system("cat $e_file | dbamon_mailer -s 'DBAmon event() $titleraw[$thishost] $e_msgid_short Suppressed - $e_msg' $debug_email") if $debug_409;
        &debug($e_h, "Events: Suppress1"); 
        return;
      }

      $server_events_this_iter{$hostname_userid[$e_h]} .= "${e_msgid_short},"; 
    }
  }

  #---- See if this event should be suppressed by the DBC Suppress_Events: parameter
  if ($suppress_events[$e_h] && index($suppress_events[$e_h],$e_msgid_short) > -1)
  {
    &logit("This event ($e_msgid_short) is specified in DBC Suppress_Events: - No notification will occur",$e_h);
    &debug($e_h, "Events: Suppress2"); 
    return;
  }

  #---- See if this event should be suppressed by the DBAMONRC Suppress_Events: parameter
  if ($g_suppress_events && index($g_suppress_events,$e_msgid_short) > -1)
  {
    &logit("This event ($e_msgid_short) is specified in DBAMONRC Suppress_Events: - No notification will occur",$e_h);
    &debug($e_h, "Events: Suppress3"); 
    return;
  }

  #---- Run the userexit (if it exists). If it ends with RC != 0, then suppress this event
  if (-x "/opt/dbamon/bin/userexit/inc_event")
  {
    require "inc_event";
    $rc = &inc_event;
    if ($rc != 0) 
    {
      &logit("event - This event ($e_msgid_short) will not be created because userexit inc_event ended with RC=$rc",$e_h);
      &debug($e_h, "Events: SuppressByUserexit"); 
      return;
    }
  }

  #---- Is this Event SNOOZED? - Start
  ($es_rows) = &readEventSnooze($e_h); 
  &debug($e_h, "readEventSnooze - es_rows=$es_rows");
  if ($es_rows > 0) 
  {
    $zmsg = "FromDB: NOT Performing Notification for Snoozed Event - es_ts=$es_ts es_user=$es_user es_reason=$es_reason - es_rows=$es_rows";
    &logit($zmsg, $e_h);

    #---- Insert message to repository (dbamon.events)
    $ticket_id = "SN";
    &eventHistory($e_msg,$e_h);

    #---- Write notification list to event log
    #$event_notify_text .= "- $zmsg\n"; 
    #&log_notify($event_notify_text,$e_h);

    &debug($e_h, "Events: Snoozed"); 
    return;
  }
  #---- Is this Event SNOOZED? - End

  #---- Settings from the SEV
  if ($e_sev eq "C")
  {
    $e_str = "!!!";
    $e_color = "red";
    $e_text = "Critical";
    $m_critical[$e_h]++;
    ##print "e_msg=$e_msg e_sev=$e_sev e_short=$e_short\n";
  }
  elsif ($e_sev eq "W")
  {
    $e_str = "***";
    $e_color = "goldenrod";
    $e_text = "Warning";
    $m_warning[$e_h]++;
  }
  elsif ($e_sev eq "I")
  {
    $e_str = "...";
    $e_color = "slateblue";
    $e_text = "Info";
    $m_info[$e_h]++ if $e_msgid ne "DBA301";
  }
  elsif ($e_sev eq "U")
  {
    $e_str = "---";
    $e_color = "green";
    $e_text = "User";
    $m_user[$e_h]++;
  }
  elsif ($e_sev eq "S")
  {
    $e_str = "###";
    $e_color = "purple";
    $e_text = "Security";
    $m_sec[$e_h]++;
  }
  elsif ($e_sev eq "P")
  {
    $e_str = "===";
    $e_color = "blue";
    $e_text = "Perf";
    $m_perf[$e_h]++;
  }
  elsif ($e_sev eq "N")
  {
    $e_str = "\@\@\@"; 
    $e_color = "black";
    $e_text = "Notation";
    $m_note[$e_h]++;
  }
  else
  {
    &error_msg("Invalid severity: $e_sev passed to /opt/dbamon/lib/event.pl - e_h=$e_h");
  }

  &eventDesc; 
  
  $z = "";
  $z = "${routine}() " if $routine ne "";
  my $e_msg_nospecialchars = "${z}$e_msg";

  $e_msg_short = "${z}$e_msg";
  $e_msg = "$e_str ${z}$e_msg $e_str";
  $e_msgsave = $e_msgid;

  #---- We need better text if SLA is NULL
  if ($dbc_parm[$e_h]{'sla'} eq "") { $z_sla = "(Null)";  } 
  else                              { $z_sla = $dbc_parm[$e_h]{'sla'}; } 
  ##print "DEBUG-EV: titleraw=$titleraw[$e_h] z_sla=($z_sla) dbc_parm{sla}=$dbc_parm[$e_h]{'sla'}\n"; 

  #---- Old
  ##$e_msgmod = "DBAmon | $titleraw[$e_h] ($id2[$e_h]/$sla[$e_h]) (EvCat=$e_cat) | ${e_msgid}: $e_msg";
  #---- New
  $e_msgmod = "DBAmon | $titleraw[$e_h] (Sev=$e_sev SLA=$z_sla) | ${e_msgid}: $e_msg_short | (EvCat=$e_cat)";

  #---- Check for event overrides
  &eventOCheck($e_msgid_short);
  ##print "Back from eventOCheck - $e_msgid_short - ov_ticket=$ov_ticket ov_pager=$ov_pager ov_email=$ov_email \n";

  if (length("$ov_ticket$ov_pager$ov_email") > 0)
  {
    &logit("Event_Override: ov_ticket=$ov_ticket ov_pager=$ov_pager ov_email=$ov_email",$e_h);
  }

  #---- Write to www event log for this instance
  &eventLog($e_msg,$e_h);
  if ($e_file ne "") { &log_file($e_file, $e_h, $e_long, $e_long_rows, $e_long_bytes); }
  &log_snooze($e_h); 

  #---- Write event text to Day log
  &logit("$e_msgid $e_text Event: $e_msg",$e_h);
  if ($e_file ne "") 
  {
    #---- Old
    ## $z1 = `cat $e_file | egrep -a -v '$e_file_grep_exclude'`;
    ## $z2 = "Event Details: \n-----(Event Details Start)-----\n$z1\n------(Event Details End)------"; 
    #---- New
    my $z2 = "Event Details: \n-----(Event Details Start)-----\n$e_long\n------(Event Details End)------"; 
    &logit($z2,$e_h);
  }

  #---- Add to the list of short messages for this instance
  if ($e_short ne "")
  {
    my $z = "<font color=$e_color>($e_short)</font>";
    #---- Only add this to the WWW list if it is NOT there already
    if (index($msgs[$e_h],$z) == -1) 
    { 
      if (length($msgs[$e_h]) == 0) 
      { 
        $msgs[$e_h] .= "$z";
      }
      else
      { 
        $msgs[$e_h] .= "<br>$z";
      }
    }
  }

  #---- The var "cache_savemsg" will be used for the DB BASED calls
  $timenum = time; 
  $z_host     = $titleraw[$e_h];
  $z_userlist = $userlist[$h_userlist[$e_h]];
  $z_msg      = $e_msg_nospecialchars;
  $cache_savemsg = "db=$z_host m=$z_msg";
  $event_cksum = &cksum($cache_savemsg); 
  if (length($cache_savemsg) > 250) { $cache_savemsg = substr($cache_savemsg,0,250); }

  #---- DB BASED DUP CHECKING:
  my ($edc_found, $edc_iters_ago, $edc_hours_ago) = &eventDupCheckDB($cache_savemsg); 
  &debug($e_h, "DEBUG_EV_FromDB: edc_found=$edc_found edc_iters_ago=$edc_iters_ago edc_hours_ago=$edc_hours_ago");
  &logit("DEBUG_EV_FromDB: edc_found=$edc_found edc_iters_ago=$edc_iters_ago edc_hours_ago=$edc_hours_ago",$e_h);
  
  #---- Insert the row from THIS ITERATION
  if ($event_cache_to_db)
  {
    $sql  = "insert into dbamon.task_event_cache 
      (te_dbamonmode,
       te_iter_seq,
       te_event_cksum,
       te_event_text, 
       te_ts_num) 
    values
      ('$modeout',
       $g_thisiter_seq, 
       $event_cksum, 
       '$cache_savemsg', 
       $timenum)
    ";
    &runSQL($sql);
  }

  #---- FromDB: Eliminate duplicates
  if ($suppress_dups_from_db == 1 && $edc_found == 1 && $edc_iters_ago <= 2) 
  {
    &logit("FromDB: NOT Performing Notification for Consecutive Duplicate Message - edc_found=$edc_found edc_iters_ago=$edc_iters_ago edc_hours_ago=$edc_hours_ago",$e_h);

    if ($g_duplicates_to_events_table ne "N")
    {
      #---- Insert message to repository (dbamon.events)
      $ticket_id = "SD";
      &eventHistory($e_msg,$e_h);
    }

    #---- Write notification list to event log
    $event_notify_text .= "- FromDB: Not performing notification for consecutive-iteration duplicate event - edc_found=$edc_found edc_iters_ago=$edc_iters_ago edc_hours_ago=$edc_hours_ago\n"; 
    &log_notify($event_notify_text,$e_h);

    &debug($e_h, "Events: Suppress4"); 
    return;
  }

  if ($event_cache_to_file)
  {
    #---- Save this message for the next iteration - prevent consecutive dup messages
    
    chomp($z_msg      = $e_msg_nospecialchars);
    chomp($z_host     = $titleraw[$e_h]);
    chomp($z_userlist = $userlist[$h_userlist[$e_h]]);
    $savemsg = "host=$z_host userlist=$z_userlist msg=$z_msg";
    system("echo '$savemsg' >> $tmpdir/dbamon_savemsg_next");   # Hmmmm
  }

  if ($suppress_dups_from_db == 0)
  {
    #---- FILE BASED DUP CHECKING:
    #---- 
    #---- See if this message is already in the SAVEMSG file - If it is, it was created
    #----  the last iteration, in which case we won't notify again.
    if (system("grep '$savemsg' $tmpdir/dbamon_savemsg_this >/dev/null") == 0)
    {
      $duplicate_flag = 1;

      ##print "Debug Info:\n";
      ##print "grep '$savemsg' $tmpdir/dbamon_savemsg_this 2>&1\n";
      ##system("grep '$savemsg' $tmpdir/dbamon_savemsg_this 2>&1");
    }
    else
    {
      $duplicate_flag = 0;
    }

    #---- FromFile: Eliminate duplicates
    if ($duplicate_flag)
    {
      &logit("FromFile: NOT Performing Notification for Consecutive Duplicate Message",$e_h);
      &debug($e_h, "FromFile: NOT Performing Notification for Consecutive Duplicate Message");

      if ($g_duplicates_to_events_table ne "N")
      {
	#---- Insert message to repository (dbamon.events)
	$ticket_id = "SD";
	&eventHistory($e_msg,$e_h);
      }

      #---- Write notification list to event log
      $event_notify_text .= "- FromFile: Not performing notification for consecutive-iteration duplicate event\n"; 
      &log_notify($event_notify_text,$e_h);

      &debug($e_h, "Events: Suppress - Consecutive"); 
      return;
    }
  }  #---- End of if  $suppress_dups_from_db == 0)

  #---- See if this event is INHIBITED 
  # MSSQL@BillB: /opt/dbamon/adm/inhibit ==> cat Event_DOW_Hour.txt
  # Event=DBA651 DOW=Thu Hour=14
  my $inhibit_file_event_dow_hour = "/opt/dbamon/adm/inhibit/Event_DOW_Hour.txt"; 
  if (-e "$inhibit_file_event_dow_hour")
  {
    my $nowhour = substr(&getdate,11,2);
    my $nowday  = substr((localtime ()), 0, 3);
    my $event_string = "Event=$e_msgid DOW=$nowday Hour=$nowhour"; 

    &debug($e_h, "DEBUG_EV: Looking for $event_string in $inhibit_file_event_dow_hour");
    $rc = &mygrep($event_string, "$inhibit_file_event_dow_hour", "$tmpfl1");
    &debug($e_h, "DEBUG_EV:   Resulting rc=$rc"); 

    if ($rc > 0)
    {
      &logit("NOT Performing Notification for Inhibitied-DOW-Hour Event",$e_h);
      
      #---- Insert message to repository (dbamon.events)
      $ticket_id = "SI";
      &eventHistory($e_msg,$e_h);

      #---- Write notification list to event log
      $event_notify_text .= "- Not performing notification for Inhibited-DOW-Hour Event\n"; 
      &log_notify($event_notify_text,$e_h);

    &debug($e_h, "Events: Suppress - Inhibit-DOW-Hour"); 
      return;
    }
  }
  
  #---- tmpfl5 will contain the header and text of message
  system("rm $tmpfl5 2>/dev/null");
  $date = &getdate();
  open(TMPFL5,  ">$tmpfl5") || &error_msg("event(#1) - Can't open output $tmpfl5 - $!");
 
  $mailformat = "html";   # html or text

  if ($mailformat eq "text")
  {
    print TMPFL5 "--------------- DBAmon Event Notification ---------------\n";
    print TMPFL5 "  Instance: $title[$e_h]\n";
    print TMPFL5 "      DBMS: $dbms[$e_h]\n";
    print TMPFL5 " Timestamp: $date\n";
    print TMPFL5 "  Severity: $e_text\n";
    print TMPFL5 "     Event: <font color=$e_color><b>$e_msg</b></font>\n";
    print TMPFL5 "---------------------------------------------------------\n";
    if ($e_msgsave ne "") 
    {
      print TMPFL5 "\nFor more info on this event, see: <a href='$dbamon_home/errors.shtml#$e_msgid_short'<b>DBAmon Event Reference: $e_msgsave</b></a>\n";
    }
    if ($e_file ne "") 
    {
      print TMPFL5 "<hr>";
      print TMPFL5 "<b>Diagnostic Data ($e_file):</b><br>\n";

      #---- NEW
      print TMPFL5 $e_long; 
      print TMPFL5 "<hr>";
    }
    $htmlsuf = " ";
  }
  else
  {
    print TMPFL5 "</pre> 
<font face=$fontdq size=2>
<table border=0 width=95%>
<tr bgcolor=darkblue><td><center><font color=white size=6 face=$fontdq>DBAmon</font></center>
<tr bgcolor=wheat>   <td><center><font face=$fontdq><b>Event Notification</b></center>
</table>
<font face=$fontdq>
    <p>
    <hr>
    $event_desc
    <br>&nbsp; 
    <table border=0 width=95%> 
      <tr bgcolor=darkblue> 
	<th colspan=2 align=left><span $style_med> &nbsp; &nbsp; <font color=wheat> Event Details: </font>
  <tr>
    <td align=right><span $style_med> Event MsgId: &nbsp; 
    <td><span $style_med> <b> <font color=$e_color> $e_msgsave  </font> <font color=black> </b>
  <tr>
    <td align=right><span $style_med> Event Short Text: &nbsp; 
    <td><span $style_med> <b> <font color=$e_color> $e_msg  </font> <font color=black> </b>
  <tr>
    <td align=right><span $style_med> Severity: &nbsp; 
    <td><span $style_med> <b> <font color=$e_color> $e_text </font> <font color=black> </b>
  "; 

    if ($e_cat ne "") 
    {
      print TMPFL5 "
      <tr>
	<td align=right><span $style_med> Event Category: &nbsp; 
	<td><span $style_med> <b> $e_cat </b>
      ";
    }

    if ($e_msgsave ne "") 
    {
      print TMPFL5 
      "<tr> 
         <td align=right><span $style_med> Event URL: 
         <td><span $style_med> <a href='$dbamon_home/errors.shtml#$e_msgid_short'> <b> DBAmon Event Reference: $e_msgsave</b></a> (Go here for help with this DBAmon Event)
      ";
    }

    print TMPFL5 "
  <tr> 
    <td width=$zleftwidth align=right><span $style_med> Instance: &nbsp; 
    <td><span $style_med> <b> $title[$e_h] </b>
  <tr>
    <td align=right><span $style_med> DBMS: &nbsp; 
    <td><span $style_med> <b> $dbms[$e_h] </b>
  <tr>
    <td align=right><span $style_med> Timestamp: &nbsp; 
    <td><span $style_med> <b> $date </b>
  <tr>
    <td align=right><span $style_med> DBAmon Mode: &nbsp; 
    <td><span $style_med> <b> $modeout ($iter_count) </b>

    </table>";

    #---- URLs for this instance
    $ztitle = $title[$e_h];
    $ztitle =~ s/ //g;
    if ($arg eq "test") { $zurl = "$g_www_url/dbamon/details_test/$ztitle.html" }
    else                { $zurl = "$g_www_url/dbamon/details/$ztitle.html" }
    if ($arg eq "test") { $zurl2 = "$g_www_url/cgi-bin/dbamon_hist.pl?$ztitle,test" }
    else                { $zurl2 = "$g_www_url/cgi-bin/dbamon_hist.pl?$ztitle" }

    #print "zurl=$zurl\n"; 
    #print "zurl2=$zurl2\n"; 

    print TMPFL5 "
    <pre>&nbsp;</pre>
    <table border=0 width=95%> 
      <tr bgcolor=darkblue> 
	<th colspan=2 align=left><span $style_med> &nbsp; &nbsp; <font color=wheat> DBAmon URLs For This Instance: </font>
      <tr bgcolor=white>
	<td width=$zleftwidth align=right><span $style_med>  Instance Details:  &nbsp;
	<td align=left><span $style_med> <a href='$zurl'> <b> $ztitle Instance Details </b> </a>
    "; 
  
    if ($dbms[$e_h] eq "MSSQL") 
    {
      print TMPFL5 "
      <tr bgcolor=white>
	<td align=right><span $style_med>  Instance Perf.:  &nbsp;
	<td align=left><span $style_med> <a href='$zurl#sqlPerf'> <b> $ztitle MSSQL Performance Readings</b> </a>
      ";
    }

    print TMPFL5 "
      <tr bgcolor=white>
	<td align=right><span $style_med>  Event History:  &nbsp;
	<td align=left><span $style_med> <a href='$zurl2'> <b> $ztitle Event History </b> </a>
    </table> 
    ";

    if ($e_file ne "") 
    {
      ##print TMPFL5 "<b>Diagnostic Data ($e_file):</b><pre>\n";
      ##print TMPFL5 "<b>Diagnostic Data For This Event:</b><pre>\n";

      print TMPFL5 "
    <font face=$fontdq>
    <pre>&nbsp;</pre>
    <table border=0 width=95%> 
      <tr bgcolor=darkblue> 
	<th colspan=2 align=left><span $style_med> &nbsp; &nbsp; <font color=wheat> <font face=$fontdq> Diagnostic Data / Event Details for This Event: </font>
    </table>
    <pre>\n"; 

      ##print TMPFL5 "<b>+=================== Event Details (Start) ===================+</b>\n\n";
      print TMPFL5 $e_long; 
      ##print TMPFL5 "\n<b>+==================== Event Details (End) ====================+</b>\n";
      print TMPFL5 "</pre>";
    }

    #---- New graph (chart) email logic
    if ($ev_chart_file && $e_sev eq "P") 
    {
      print TMPFL5 "
    <font face=$fontdq>
    <pre>&nbsp;</pre>
    <table border=0 width=95%> 
      <tr bgcolor=darkblue> 
	<th colspan=2 align=left><span $style_med> &nbsp; &nbsp; <font color=wheat> 
          <font face=$fontdq> Performance Metric $ev_perf_metric Recent History Chart (inline or attached depending on your mail client): </font>
    </table>
      <img src='cid:$ev_chart_file' style='border:2px'/>
      "; 

      $ev_attachments = "-a /opt/dbamon/htm/graphs/dyn/$ev_chart_file "; 
      $ev_chart_file = ""; 
      $ev_perf_metric = ""; 
    }

    $htmlsuf = " -h";
  }

  print TMPFL5 "<hr> <pre> &nbsp; </pre>\n";
  close(TMPFL5);

  #---- TEST
  #system("cp $tmpfl5 /tmp/z");

  #---- See if this event was specified in DBC No_Tickets_For_Events: PARM
  if (index($dbc_NTFE_events[$e_h],$e_msgid_short) > -1)
  {
    &logit("Ticket will not be created - This event ($e_msgid_short) WAS specified in DBC No_Tickets_For_Events: parm",$e_h);
    $event_notify_text .= "- Ticket not created since this event ($e_msgid_short) WAS specified in DBC No_Tickets_For_Events: parm\n"; 
    $ticket_suppress_NTFE = 1; 
  }
  else
  {
    $ticket_suppress_NTFE = 0; 
  }

  #---- Create ticket if all planets are in alignment
  #----    EV_TICKET_CHECK: e_sev=C make_ticket=Y ov_ticket= e_notix= ticket_suppress_ntfe=0
  &debug($e_h, "EV_TICKET_CHECK: e_sev=$e_sev make_ticket=$make_tickets mth=$make_tickets_dbc[$e_h] ov_ticket=$ov_ticket e_notix=$e_notix ticket_suppress_ntfe=$ticket_suppress_NTFE"); 
  if (
      $e_sev eq "C" && 
      $make_tickets eq "Y" && 
      $make_tickets_dbc[$e_h] ne "N" && 
      -x "/opt/dbamon/bin/userexit/ticket" && 
      $ov_ticket ne "N" && 
      $e_notix ne 'NOTIX' &&
      $ticket_suppress_NTFE == 0 
     )
  {
    ##&logit("  Invoking userexit /opt/dbamon/bin/userexit/ticket to create ticket - ERRMSG=$ERRMSG INSTANCE=$title[$e_h]",$e_h);
    &debug($e_h, "EV_TICKET_CHECK:   IN MAKE TICKET"); 
    system("rm $tmpfl8 2>/dev/null");
    $ENV{"INSTANCE"}=$title[$e_h];
    $ENV{"ERRFILE"} =$tmpfl5;
    $ENV{"ERRMSG"}  =$e_msgmod;
    $ENV{"DBMS"}    =$dbms[$e_h];
    $ENV{"HOST"}    =$hostfix[$e_h];
    $ENV{"ID1"}     =$id1[$e_h];
    $ENV{"ID2"}     =$id2[$e_h];
    $ENV{"ID3"}     =$id3[$e_h];
    $ENV{"ID4"}     =$id4[$e_h];
    $ENV{"ID5"}     =$id5[$e_h];
    $ENV{"ID6"}     =$id6[$e_h];
    $ENV{"ID7"}     =$id7[$e_h];
    $ENV{"ID8"}     =$id8[$e_h];
    $ENV{"ID9"}     =$id9[$e_h];
    $ENV{"ID10"}    =$id10[$e_h];
    $ENV{"ID11"}    =$id11[$e_h];
    $ENV{"ID12"}    =$id12[$e_h];
    $ENV{"ID13"}    =$id13[$e_h];
    $ENV{"ID14"}    =$id14[$e_h];
    $ENV{"ID15"}    =$id15[$e_h];
    $ENV{"ID16"}    =$id16[$e_h];
    $ENV{"ID17"}    =$id17[$e_h];
    $ENV{"ID18"}    =$id19[$e_h];
    $ENV{"ID19"}    =$id19[$e_h];
    $ENV{"ID20"}    =$id20[$e_h];
    $ENV{"SEV"}     =$e_sev;
    $ENV{"MSGID"}   =$e_msgid;
    $ENV{"LONG"}    =$e_long;
    $ENV{"SHORT"}   =$e_short;
    $ENV{"MODE"}    =$modeout;
    $ENV{"CAT"}     =$e_cat;
    $ENV{"RESULTS_FILE"} =$tmpfl8;
    $ENV{"SLA"}     =$dbc_parm[$e_h]{'sla'};

    if ($e_pri_override) 
    {
      $ENV{"PRIORITY"} = $e_pri_override;
    }
    else
    {
      $ENV{"PRIORITY"}=$priority[$e_h];
    }

    if ($pager_flag[$e_h] && $g_pager eq "Y" && $e_sev eq "C" && $ov_pager ne "N")
    {
      $ENV{"PAGER"} ="Y";
    }
    else
    {
      $ENV{"PAGER"} ="";
    }

    $ticket_id = "Y";

    #system("rm $tmpfl8 2>/dev/null");
    unlink "$tmpfl8";

    $event_notify_text .= "- Created TICKET (invoked /opt/dbamon/bin/userexit/ticket)\n"; 

    #---- Old
    ##$rc = system("/opt/dbamon/bin/userexit/ticket");
    #---- New
    my $ticket_log = "/opt/dbamon/log/event_ticket_$dow.txt"; 
    &logit("  Event: Sending TICKET - Running: /opt/dbamon/bin/userexit/ticket >> $ticket_log", "", "", 1);
    &logit("    $titleraw[$e_h] | $e_msg", "", "", 1);
    $rc = system("/opt/dbamon/bin/userexit/ticket >> $ticket_log 2>&1");

    if (-e "$tmpfl8")
    {
      chop($ticket_id = `cat $tmpfl8`);
      &logit("Resulting ticket_id=$ticket_id",e_h);
    }
  }
  #push @eventdb_ticket, $ticket_id;

  #---- Send PAGE 

  $page_sent = "";

  if ($pager_flag[$e_h] && $g_pager eq "Y" && $e_sev eq "C" && $ov_pager ne "N")
  {
    $page_sent = "Y";
    
    #---- Optionally invoke userexit 
    if (-x "/opt/dbamon/bin/userexit/pager")
    {
      &logit("Invoking userexit: pager");
      my $pager_log = "/opt/dbamon/log/event_pager_$dow.txt"; 

      $ENV{"INSTANCE"}=$title[$e_h];
      $ENV{"D_HOST"}  =$title[$e_h];
      $ENV{"ERRFILE"} =$tmpfl5;
      $ENV{"ERRMSG"}  =$e_msgmod;
      $ENV{"D_MSG"}   =$e_msgmod;
      $ENV{"D_FILE"}  =$e_file;
      $ENV{"DBMS"}    =$dbms[$e_h];
      $ENV{"HOST"}    =$hostfix[$e_h];
      $ENV{"ID1"}     =$id1[$e_h];
      $ENV{"ID2"}     =$id2[$e_h];
      $ENV{"ID3"}     =$id3[$e_h];
      $ENV{"ID4"}     =$id4[$e_h];
      $ENV{"ID5"}     =$id5[$e_h];
      $ENV{"ID6"}     =$id6[$e_h];
      $ENV{"ID7"}     =$id7[$e_h];
      $ENV{"ID8"}     =$id8[$e_h];
      $ENV{"ID9"}     =$id9[$e_h];
      $ENV{"ID10"}    =$id10[$e_h];
      $ENV{"ID11"}    =$id11[$e_h];
      $ENV{"ID12"}    =$id12[$e_h];
      $ENV{"ID13"}    =$id13[$e_h];
      $ENV{"ID14"}    =$id14[$e_h];
      $ENV{"ID15"}    =$id15[$e_h];
      $ENV{"ID16"}    =$id16[$e_h];
      $ENV{"ID17"}    =$id17[$e_h];
      $ENV{"ID18"}    =$id18[$e_h];
      $ENV{"ID19"}    =$id19[$e_h];
      $ENV{"ID20"}    =$id20[$e_h];
      $ENV{"SEV"}     =$e_sev;
      $ENV{"MSGID"}   =$e_msgid;
      $ENV{"LONG"}    =$e_long;
      $ENV{"SHORT"}   =$e_short;
      $ENV{"MODE"}    =$modeout;
      $ENV{"CAT"}     =$e_cat;
      $ENV{"SLA"}     =$dbc_parm[$e_h]{'sla'};
      ##system("export D_HOST=$title[$e_h];export D_MSG='$e_msg';export D_FILE=$e_file;/opt/dbamon/bin/userexit/pager");

      &logit("  Event: Sending PAGE - Running: /opt/dbamon/bin/userexit/pager >> $pager_log", "", "", 1);
      &logit("    $titleraw[$e_h] | $e_msg", "", "", 1);

      #---- Old
      ##$rc = system("/opt/dbamon/bin/userexit/pager");
      #---- New
      $rc = system("/opt/dbamon/bin/userexit/pager >> $pager_log 2>&1");

      &logit("Back from userexit: pager - rc=$rc");
    }

    #---- Obsolete - Was used for thecompany
    ## Customization for 1/2 company 
    #$mx = sprintf("Sending pager notification Subject=%s EMail=%s",$pager_subject[$e_h],$pager_email[$e_h]);
    #&logit($mx,$e_h);
    #if (($company eq "thecompany2" || $company eq "thecompany") && $pager_subject[$e_h] eq "DUMMY")
    #{
    #  &logit("Invoking page_dbagap for 1/2 company",$e_h);
    #  $cmd = "echo '(DBAmon) $titleraw[$e_h] $e_msg' | /usr/local/sap/tools/page_dbagap";
    #  $rc = system($cmd);
    #  if ($debug_crit)
    #  {
    #    printf("page_dbagap rc=%s cmd=%s\n",$rc,$cmd);
    #  }
    #}

    #---- Send page IF DBC Pager_EMail: was specified
    #----   Note that it may be that ID4=platinum and the only pager action was done by the userexit logic above. 
    #----   In that case, pager_email will be DUMMY and this logic won't happen.
    if ($pager_email[$e_h] ne "" and $pager_email[$e_h] ne "DUMMY")
    {
      my $email_log = "/opt/dbamon/log/event_email_$dow.txt"; 
      $mx = sprintf("Sending PAGER EMail to %s",$pager_email[$e_h])
      &logit($mx,$e_h);

      if ($pager_subject[$e_h] eq "") 
      {
        $pager_subject[$e_h] = "DBAmon Notification";
      }

      &logit("  Event: Sending EMAIL-1 - Running: /opt/dbamon/bin/userexit/pager >> $pager_log", "", "", 1);
      &logit("    $titleraw[$e_h] | $e_msg", "", "", 1);

      $cmd = "echo '(DBAmon) [$id3[$e_h]] $titleraw[$e_h]-$e_msg ' | dbamon_mailer -m -s '$pager_subject[$e_h]' $pager_email[$e_h]";
      #---- Old
      ##$rc = system($cmd);
      #---- New
      $rc = system("$cmd >> $email_log 2>&1");

      $event_notify_text .= "- PAGE Sent to $pager_email[$e_h]\n"; 
    }
  }

  #---- If this host uses OpC, send opcmsg - OBSOLETE

  if ($opc_appl[$e_h] ne "" && $opc eq "Y")
  {
    $msg_fix = $e_msg;
    $msg_fix =~ s/!//g; 

    &logit("Sending OpC Message",$e_h);

    $cmd = "remsh $opc_gw[$e_h] '/opt/OV/bin/OpC/opcmsg severity=$e_text application=\"$opc_appl[$e_h]\" object=$opc_obj[$e_h] msg_text=\"$msg_fix $opc_trailer\" msg_grp=$opc_mg[$e_h]'";

    if ($debug_opc || $debug_crit) 
    {
      printf("About to execute: %s\n",$cmd);
    }
    $rc = system($cmd);
    if ($rc != 0) 
    {
      printf("Command rc=%s\n",$rc);
    }
  }

  #---- "Hard-Coded" email messages

  ##print "e_msgid=$e_msgid\n";
  if ($e_msgid eq "DBA238W")
  {
    $email_address = "aaa-bbb-ccc-hosting\@thecompany.com";
    $cmd = "cat $tmpfl5 | dbamon_mailer $htmlsuf -m -s '$e_msgmod' $email_address";
    $rc = system($cmd);
    &logit("Sent Hard-Coded EMail to $email_address for event $e_msgid - rc=$rc",$e_h);
  }

  #---- See if this sev was set to EMAIL by DBC Event_EMail_For_Sevs: was specified
  ##print "DEBUG event - eefs=$dbc_event_email_for_sevs[$e_h]\n"; 
  if (index($dbc_event_email_for_sevs[$e_h], $e_sev) > -1)
  {
    &logit("This event sev ($e_sev) IS covered by DBC Event_EMail_For_Sevs:",$e_h);
    ##print "DEBUG event - eefs HIT\n"; 
    $dbc_event_email = 1;
  }
  else
  {
    $dbc_event_email = 0; 
  }

  #---- Send EMAIL notification
  ##if ($e_sev eq "C" || $e_sev eq "U" || $ov_email eq "Y" || $dbc_event_email)
  if ($e_sev eq "S" || $e_sev eq "C" || $e_sev eq "U" || $ov_email eq "Y" || $dbc_event_email)
  {
    ##print "hu=($h_userlist[$e_h])\n";

    if (uc $ov_email eq "N") 
    {
      $event_notify_text .= "- EventOverride ov_email=N - Not sending EMAIL notification\n"; 
    }
    elsif ($h_userlist[$e_h] eq "[CMDB]")
    {
      &eventNotifyCMDB;
      $event_notify_text .= "- EMAIL Sent to CMDB contacts\n"; 
    }
    elsif ($h_userlist[$e_h] eq "[CMDB]")
    {
      &eventNotifyCMDB;
      $event_notify_text .= "- EMAIL Sent to CMDB contacts\n"; 
    }
    elsif ($h_userlist[$e_h] ne "")
    {
      &eventNotify;
      $event_notify_text .= "- EMAIL Sent to DBC Notify_List: $h_userlist[$e_h] ($userlist[$h_userlist[$e_h]])\n"; 
    }
    else
    {
      &logit("Notify not sent - Notify_List: is null",$e_h);
      $event_notify_text .= "- EMAIL Notification not sent since DBC Notify_List: is NULL\n"; 
    }

  }

  if ($e_email ne "")
  {

    &logit("  Event: Sending EMAIL-2 - Running: /opt/dbamon/bin/userexit/pager >> $pager_log", "", "", 1);
    &logit("    $titleraw[$e_h] | $e_short", "", "", 1);
    $cmd = "cat $tmpfl5 | dbamon_mailer $htmlsuf -m -s '$e_msgmod' $e_email";
    $rc = system($cmd);

    $event_notify_text .= "- EMAIL Sent to $e_email\n"; 
  }
  
  #---- Log to history table
  if (length($e_msg) > 495)
  {
    $e_msg = substr($e_msg,0,485) . " (Truncated)";
  }

  #---- Create EVENT_HISTORY row
  if ($e_sev eq "C")
  {
    $sql  = "insert into dbamon.event_history (dbamonmode,system,title,dbms,sev,text,msgid,ticket) \n";
    $sql .= "values ('$modeout',
			 '$hostname[$e_h]',
			 '$title[$e_h]',
			 '$dbms[$e_h]',
			 '$e_sev',
			 '$e_msg',
			 '$e_msgid',
			 '$ticket_id')";
    &runSQL($sql);
  }

  #---- Insert message to repository (dbamon.events)
  &eventHistory($e_msg,$e_h);

  #---- Write notification list to event log
  &log_notify($event_notify_text,$e_h);
}

#--------------------------------------------------------------------------------
sub eventDesc
#--------------------------------------------------------------------------------
{
  if ($e_sev eq "P") 
  {
    $event_desc = "
<table border=0 bgcolor=darkblue width=95%>
  <tr bgcolor=darkblue>
    <td><span $style_med><font color=wheat>
      <b>This is a DBAmon Severity=P (Performance)
        <a style='text-decoration:underline; color:#99ffff;' href='http://dbamon.com/misc/notify.shtml'><b>Event</b></a>.
      </b> </font>
  <tr bgcolor=wheat>
    <td><span $style_med>
      It simply means that a DBAmon Performance Metric for this instance is out-of-threshold.
    This event is informational and is not necessarily <i>actionable</i>.  
    It may be entirely normal for a DBAmon Performance Metric to be out-of-threshold during a non-Prime shift. 
    If you do not wish to view DBAmon Severity=(P)erformance Events created during non-prime shifts, ask the 
    $admin_email_url to specify: 
    <b>Perf_Events_During_Shifts: P</b> in the 
    <a href='http://dbamon.com/config/dbc.shtml'><b>DBAmon DBC file</b></a> for this instance.
      If this event does not apply to this instance, then ask your $admin_email_url to suppress this event for this instance
      using the <b>Suppress_Events:</b> parameter in the  
      <a href='http://dbamon.com/config/dbc.shtml'><b>DBAmon DBC file</b></a> for this instance.
</table>
  "; 
  }
  elsif ($e_sev eq "C") 
  {
    $event_desc = "
<table border=0 bgcolor=darkblue width=95%>
  <tr bgcolor=darkblue>
    <td><span $style_med><font color=wheat>
      <b>This is a DBAmon Severity=C (Critical) 
        <a style='text-decoration:underline; color:#99ffff;' href='http://dbamon.com/misc/notify.shtml'><b>Event</b></a>.
      </b> </font>
  <tr bgcolor=wheat>
    <td><span $style_med>
      It means that a DBAmon has detected a condition with this instance which is
    critical and is probably <i>actionable</i>.  
      If this event does not apply to this instance, then ask your $admin_email_url suppress this event for this instance
      using the <b>Suppress_Events:</b> parameter in the  
      <a href='http://dbamon.com/config/dbc.shtml'><b>DBAmon DBC file</b></a> for this instance.
</table>
  "; 
  }
  elsif ($e_sev eq "W") 
  {
    $event_desc = "
<table border=0 bgcolor=darkblue width=95%>
  <tr bgcolor=darkblue>
    <td><span $style_med><font color=wheat>
      <b>This is a DBAmon Severity=W (Warning) 
        <a style='text-decoration:underline; color:#99ffff;'  href='http://dbamon.com/misc/notify.shtml'><b>Event</b></a>.
      </b> </font>
  <tr bgcolor=wheat>
    <td><span $style_med>
      It means that a DBAmon has detected a condition with this instance where we are approaching the Critical threshold
      and therefore is
      is probably <i>actionable</i>.  
      If this event does not apply to this instance, then ask your $admin_email_url to suppress this event for this instance
      using the <b>Suppress_Events:</b> parameter in the  
      <a href='http://dbamon.com/config/dbc.shtml'><b>DBAmon DBC file</b></a> for this instance.
</table>
  "; 
  }
  elsif ($e_sev eq "U") 
  {
    $event_desc = "
<table border=0 bgcolor=darkblue width=95%>
  <tr bgcolor=darkblue>
    <td><span $style_med><font color=wheat>
      <b>This is a DBAmon Severity=U (User) 
        <a style='text-decoration:underline; color:#99ffff;'  href='http://dbamon.com/misc/notify.shtml'><b>Event</b></a>.
      </b> </font>
  <tr bgcolor=wheat>
    <td><span $style_med>
      It means that a DBAmon has detected a condition with this instance where something about the (U)sage of this instance 
      is unusual. For example, there may be a job which has been running for at least 12 hours. 
      However, this may be completely normal for your environment. Also, DBAmon <i>Housekeeping</i> events use the (U)ser Event Severity. 
      Some examples of these are Jobs not owned by SA, Instance with no User Databases, etc.. 
      Whether these events are 
      <i>actionable</i> depends on your environment.  
      If this event does not apply to this instance, then ask your $admin_email_url to suppress this event for this instance
      using the <b>Suppress_Events:</b> parameter in the  
      <a href='http://dbamon.com/config/dbc.shtml'><b>DBAmon DBC file</b></a> for this instance.
</table>
  "; 
  }
  elsif ($e_sev eq "S") 
  {
    $event_desc = "
<table border=0 bgcolor=darkblue width=95%>
  <tr bgcolor=darkblue>
    <td><span $style_med><font color=wheat>
      <b>This is a DBAmon Severity=S (Security) 
        <a style='text-decoration:underline; color:#99ffff;'  href='http://dbamon.com/misc/notify.shtml'><b>Event</b></a>.
      </b> </font>
  <tr bgcolor=wheat>
    <td><span $style_med>
      It means that a DBAmon has detected a Security exception for this instance. Depending on the nature of the Event, it may
      or may not be <i>actionable</i>. 
      If this event does not apply to this instance, then ask your $admin_email_url to suppress this event for this instance
      using the <b>Suppress_Events:</b> parameter in the  
      <a href='http://dbamon.com/config/dbc.shtml'><b>DBAmon DBC file</b></a> for this instance.
</table>
  "; 
  }
  elsif ($e_sev eq "I") 
  {
    $event_desc = "
<table border=0 bgcolor=darkblue width=95%>
  <tr bgcolor=darkblue>
    <td><span $style_med><font color=wheat>
      <b>This is a DBAmon Severity=I (Informational) 
        <a style='text-decoration:underline; color:#99ffff;'  href='http://dbamon.com/misc/notify.shtml'><b>Event</b></a>.
      </b> </font>
  <tr bgcolor=wheat>
    <td><span $style_med>
      It means that a DBAmon has detected a minor exception for this instance. Depending on the nature of the Event, it is
      probably not <i>actionable</i>. 
      If this event does not apply to this instance, then ask your $admin_email_url to suppress this event for this instance
      using the <b>Suppress_Events:</b> parameter in the  
      <a href='http://dbamon.com/config/dbc.shtml'><b>DBAmon DBC file</b></a> for this instance.
</table>
  "; 
  }
  else { $event_desc = ""; }
}

#--------------------------------------------------------------------------------
1;
