sub readConfig
#--------------------------------------------------------------------------------
#
# readConfig - Read the dbamonrc parameter file
#
#--------------------------------------------------------------------------------
# Changes:
#  bb 04Jun04: Added remsh_sleep including its default value of 60.
#  bb 07Jul05: New Default_T_SGA: parameter
#  bb 10Sep12: New Quote_Domain_Users: parm
#  bb 25Nov12: New Suppress_Events: parm
#  bb 14Jul13: 5.45.A - New df_t_perfcounter logic 
#  bb 04Dec16: 5.55.D - Changed resmh_sleep default from 60 to 5
#--------------------------------------------------------------------------------
{
  &logit("Reading dbamonrc file=$parmfl");

  %config_parm = (); 

  $i_users = 0;
  $debug_read = 0;
  
  my $zwork = ""; 

  open(DBAMONRC, "$parmfl"  ) || &error_msg("Cannot open input parmfl=$parmfl - $!");
  while (<DBAMONRC>)
  {
    &parse;
    
    if ($NF == 0)
    {
      next;
    }

    #---- Try to extract Admin_EMail because we will need that for parsing
    #---- and it may be at the end of the file
    if ($F1 eq "Admin_EMail:")
    {
      $dbamon_admin = $F2;  
      $admin_email = $F2; 
      next;
    }
    if ($F1 eq "Admin_Pager:")
    {
      $admin_pager = $F2; 
      next;
    }

    if ($F1 eq "Include:")
    {
      &logit("dbamonrc Include: parameter is now obsolete - $parmdir will be searched for *.dbc files");
      next;
    }
    
    if (substr($F1,0,1) ne "#" && $F1 ne "Section")
    {

      if ($section eq "HOSTS") 
      { 
        &logit("Hosts: section of dbamonrc is now obsolete - $parmdir will be searched for *.dbc files");
      }   # End of HOSTS section

      if ($section eq "USERS") 
      {
        $i_users++;
        $thislist = $F1;
	$templist = $F0;
	$userlist[$thislist] = substr($templist,6) ;
        ##printf("Processing Userlist/User %s/%s\n",$thislist,$userlist[$thislist])
      }   # End of USERS section

      if ($section eq "SYSTEM") 
      {
        ($zwork = lc $F1) =~ s/\:$//;
        $config_parm{$zwork} = $F2;

	if    ($F1 eq "Sleeptime:")           { $sleep_time = $F2; next; }
        elsif ($F1 eq "Debug:")               { $rcdebug = $F2; next; }
        elsif ($F1 eq "Filecheck:")           { $filecheck = $F2; next; }
        elsif ($F1 eq "Iterations:")          { $iterations = $F2; next; }
        elsif ($F1 eq "Count_Tables:")        { $count_tables = $F2; next; }
        elsif ($F1 eq "Admin_EMail:")         { $dbamon_admin = $F2; next; }
        elsif ($F1 eq "Debug_EMail:")         { $debug_email = $F2; next; }
        elsif ($F1 eq "Report_EMail:")        { $report_email = "$F2 $F3 $F4 $F5 $F6 $F7 $F8 $F9"; }
        elsif ($F1 eq "Log_Path:")            { $log_path = $F2; next; }
        elsif ($F1 eq "Save_Dir:")            { $save_dir = $F2; next; }
        elsif ($F1 eq "PN_Dir:")              { $post_dir = $F2; next; }
        elsif ($F1 eq "Use_MSSQL:")           { $use_mssql = $F2; next; }
        elsif ($F1 eq "Use_MySQL:")           { $use_mysql = $F2; next; }
        elsif ($F1 eq "Use_Oracle:")          { $use_oracle = $F2; next; }
        elsif ($F1 eq "Use_CMDB:")            { $use_cmdb = $F2; next; }
        elsif ($F1 eq "Remsh_Sev:")           { $remsh_sev = $F2; next; }
        elsif ($F1 eq "Remsh_Retries:")       { $remsh_retries = $F2; next; }
        elsif ($F1 eq "Remsh_Retry_Sleep:")   { $remsh_sleep = $F2; next; }
        elsif ($F1 eq "HTML_Rpt_Path:")       { $report_path = $F2; next; }
        elsif ($F1 eq "HTML_Log_Path:")       { $html_path = $F2; next; }
        elsif ($F1 eq "HTML_Sys_Path:")       { $html_sys_path = $F2; next; }
        elsif ($F1 eq "HTML_Events_Dir:")     { $html_events_dir = $F2; next; }
        elsif ($F1 eq "HTTP_Events_Dir:")     { $http_events_dir = $F2; next; }
        elsif ($F1 eq "HTTP_Details_Dir:")    { $http_details_dir = $F2; next; }
        elsif ($F1 eq "HTTP_Log_Path:")       { $http_path = $F2 }
        elsif ($F1 eq "HTTP_Rpt_Path:")       { $http_rpt_path = $F2 }
        elsif ($F1 eq "Images_Dir:")          { $images_path = $F2 }
        elsif ($F1 eq "OpC_Messages:")        { $opc = $F2 }
        elsif ($F1 eq "OpC_Trailer:")         { $opc_trailer = substr($F0,18) }
        elsif ($F1 eq "Ver_Colors:")          { $ver_colors = $F2 }
        elsif ($F1 eq "Make_Tickets:")        { $make_tickets = $F2 }
        elsif ($F1 eq "MSSQL_Instance:")      { $mssql_instance = $F2 }
        elsif ($F1 eq "ORACLE_SID:")          { $oracle_sid = $F2 }
        elsif ($F1 eq "ORACLE_HOME:")         { $oracle_home = $F2 }
        elsif ($F1 eq "ORACLE_Password:")     { $oracle_password = $F2 }
        elsif ($F1 eq "Num_Tasks:")           { $num_tasks = $F2 }
        elsif ($F1 eq "Task_Cfg_File:")       { $task_cfg_file = $F2 }
        elsif ($F1 eq "Iter_Min_Secs:")       { $iter_min_secs = $F2 }
        elsif ($F1 eq "Paging:")              { $g_pager = $F2 }
        elsif ($F1 eq "Run_oraSec:")          { $g_orasec = $F2 }

        elsif ($F1 eq "PhoneHome_Messages:")  { $g_phonehome = $F2 }

        elsif ($F1 eq "WWW_Check:")           { $www_check = $F2 }
        elsif ($F1 eq "WWW_Font:")            { $www_font = $F2 }
        elsif ($F1 eq "WWW_Magnify:")         { $www_magnify = $F2 }
        elsif ($F1 eq "WWW_Event_Isolate:")   { $www_event_isolate = $F2 }
        elsif ($F1 eq "WWW_Table_BGColor:")       { $www_table_bgcolor = $F2 }
        elsif ($F1 eq "WWW_Table_HeadBGColor:")   { $www_table_headbgcolor = $F2 }

        elsif ($F1 eq "WWW_URL:")             { $g_www_url = $F2; }
        elsif ($F1 eq "Company_Title:")       { $g_company_title = substr($F0,15) }

        elsif ($F1 eq "ID1_Breakout:")        { $id1_breakout = $F2 }
        elsif ($F1 eq "ID2_Breakout:")        { $id2_breakout = $F2 }
        elsif ($F1 eq "ID3_Breakout:")        { $id3_breakout = $F2 }
        elsif ($F1 eq "ID4_Breakout:")        { $id4_breakout = $F2 }
        elsif ($F1 eq "ID5_Breakout:")        { $id5_breakout = $F2 }
        elsif ($F1 eq "ID6_Breakout:")        { $id6_breakout = $F2 }
        elsif ($F1 eq "ID7_Breakout:")        { $id7_breakout = $F2 }
        elsif ($F1 eq "ID8_Breakout:")        { $id8_breakout = $F2 }
        elsif ($F1 eq "ID9_Breakout:")        { $id9_breakout = $F2 }
        elsif ($F1 eq "ID10_Breakout:")       { $id10_breakout = $F2 }
        elsif ($F1 eq "ID11_Breakout:")       { $id11_breakout = $F2 }
        elsif ($F1 eq "ID12_Breakout:")       { $id12_breakout = $F2 }
        elsif ($F1 eq "ID13_Breakout:")       { $id13_breakout = $F2 }
        elsif ($F1 eq "ID14_Breakout:")       { $id14_breakout = $F2 }
        elsif ($F1 eq "ID15_Breakout:")       { $id15_breakout = $F2 }
        elsif ($F1 eq "ID16_Breakout:")       { $id16_breakout = $F2 }
        elsif ($F1 eq "ID17_Breakout:")       { $id17_breakout = $F2 }
        elsif ($F1 eq "ID18_Breakout:")       { $id18_breakout = $F2 }
        elsif ($F1 eq "ID19_Breakout:")       { $id19_breakout = $F2 }
        elsif ($F1 eq "ID20_Breakout:")       { $id20_breakout = $F2 }
        elsif ($F1 eq "ID1_Label:")           { chop($id1_label = substr($F0,index($F0,$F2))) }
        elsif ($F1 eq "ID2_Label:")           { chop($id2_label = substr($F0,index($F0,$F2))) }
        elsif ($F1 eq "ID3_Label:")           { chop($id3_label = substr($F0,index($F0,$F2))) }
        elsif ($F1 eq "ID4_Label:")           { chop($id4_label = substr($F0,index($F0,$F2))) }
        elsif ($F1 eq "ID5_Label:")           { chop($id5_label = substr($F0,index($F0,$F2))) }
        elsif ($F1 eq "ID6_Label:")           { chop($id6_label = substr($F0,index($F0,$F2))) }
        elsif ($F1 eq "ID7_Label:")           { chop($id7_label = substr($F0,index($F0,$F2))) }
        elsif ($F1 eq "ID8_Label:")           { chop($id8_label = substr($F0,index($F0,$F2))) }
        elsif ($F1 eq "ID9_Label:")           { chop($id9_label = substr($F0,index($F0,$F2))) }
        elsif ($F1 eq "ID10_Label:")          { chop($id10_label = substr($F0,index($F0,$F2))) }
        elsif ($F1 eq "ID11_Label:")          { chop($id11_label = substr($F0,index($F0,$F2))) }
        elsif ($F1 eq "ID12_Label:")          { chop($id12_label = substr($F0,index($F0,$F2))) }
        elsif ($F1 eq "ID13_Label:")          { chop($id13_label = substr($F0,index($F0,$F2))) }
        elsif ($F1 eq "ID14_Label:")          { chop($id14_label = substr($F0,index($F0,$F2))) }
        elsif ($F1 eq "ID15_Label:")          { chop($id15_label = substr($F0,index($F0,$F2))) }
        elsif ($F1 eq "ID16_Label:")          { chop($id16_label = substr($F0,index($F0,$F2))) }
        elsif ($F1 eq "ID17_Label:")          { chop($id17_label = substr($F0,index($F0,$F2))) }
        elsif ($F1 eq "ID18_Label:")          { chop($id18_label = substr($F0,index($F0,$F2))) }
        elsif ($F1 eq "ID19_Label:")          { chop($id19_label = substr($F0,index($F0,$F2))) }
        elsif ($F1 eq "ID20_Label:")          { chop($id20_label = substr($F0,index($F0,$F2))) }
        elsif ($F1 eq "Loc_Breakout:")        { $loc_breakout = $F2 }
        elsif ($F1 eq "Auto_Detect_Aliases:") { $auto_detect_aliases = $F2 }
        elsif ($F1 eq "Use_DBC_Firstchar:")   { $use_dbc_firstchar = $F2 }
        elsif ($F1 eq "Use_DNS:")             { $g_use_dns = $F2 }
        elsif ($F1 eq "Probe_Timeout:")       { $probe_timeout = $F2 }
        elsif ($F1 eq "Run_SPFile_Check:")    { $g_spfile = $F2 }
        elsif ($F1 eq "History_To_File:")     { $g_history_to_file = uc $F2 }
        elsif ($F1 eq "SQLPerf_Sev_To_Perf:") { $g_sqlperf_sev = uc $F2 }
        elsif ($F1 eq "SQL_Job_Owner_Event:") { $g_sql_job_owner_event = uc $F2 }
        elsif ($F1 eq "SQL_DB_Owner_Event:")  { $g_sql_db_owner_event = uc $F2 }

        elsif ($F1 eq "Suppress_Events:")     { $g_suppress_events = $F2; }


        elsif ($F1 eq "Quote_Domain_Users:")  { $g_quote_domain_users = uc $F2 }

        elsif ($F1 eq "Duplicates_To_Events_Table:")  { $g_duplicates_to_events_table = uc $F2; }

        #---- Throw away parms
        elsif ($F1 eq "Table_METRICS_HOURLY:")      { }
        elsif ($F1 eq "Table_Detail_Retain_Days:")  { $g_detail_retain_days = uc $F2; }

        #---- Monitoring defaults
        elsif ($F1 eq "Default_Backup_Age_Full_Days:")  { $df_mssql_backup_age_full = $F2; }
        elsif ($F1 eq "Default_MSSQL_Job_Fail_Check:")  { $df_mssql_job_fail_check = $F2; }
        elsif ($F1 eq "Default_MSSQL_Log_Min_Sev:")     { $df_mssql_log_min_sev = $F2; }
        elsif ($F1 eq "Default_DBMS:")        { $df_dbms = $F2; }
        elsif ($F1 eq "Default_BA_Arc:")      { $df_ba_arc = $F2; }
        elsif ($F1 eq "Default_Min_CF:")      { $df_t_cf_min = $F2; }

        elsif ($F1 eq "Default_T_MSSQL_VLF_Count:")       { $df_t_mssql_vlf_count = $F2; }

        elsif ($F1 eq "Default_MSSQL_Log_Exclude_Strings:")
        {
          $df_mssql_log_exclude_strings = &parseQuoted($F0);
        }

        #---- New way
        elsif ($F1 eq "MSSQL_Svc_Acct_Check:")              { }

        elsif ($F1 eq "Default_MSSQL_Tran_Duration_Hrs:")   { }
        elsif ($F1 eq "Default_MSSQL_Tran_TLOG_GB:")        { }
        elsif ($F1 eq "Default_SLA:")                       { }

        elsif ($F1 eq "Default_T_SA_Max_Logins:")           { $df_t_sa_max_logins = $F2; }

        elsif ($F1 eq "Default_Instance_Dur_Max_Secs:")     { }

        elsif ($F1 eq "Default_T_SGA:")
        {
          @words = split(/\//,$F2);
          $df_t_sga_w = $words[0];
          $df_t_sga_c = $words[1];
          ##print "df_t_sga_w[$i_hosts]=$df_t_sga_w[$i_hosts]\n";
          next;
        }
        elsif ($F1 eq "Default_T_NUSERPROC:")
        {
          @words = split(/\//,$F2);
          $df_t_nuserproc_w = $words[0];
          $df_t_nuserproc_c = $words[1];
          &debug($thishost, "df_t_nuserproc_w=$df_t_nuserproc_w");
          &debug($thishost, "df_t_nuserproc_c=$df_t_nuserproc_c");
          next;
        }
        elsif ($F1 eq "Default_T_DB_Cache:")
        {
          @words = split(/\//,$F2);
          $df_t_db_cache_c = $words[0];
          $df_t_db_cache_w = $words[1];
          next;
        }
        elsif ($F1 eq "Default_T_PerfCounter:")
        {
          my $zcounter = uc $F2;
          my @zwords = split(/\//,$F3);
          my $zwarn  = $zwords[0];
          my $zcrit  = $zwords[1];
          my $zoperator = uc $F4;
          $zoperator = "GE" if ! $zoperator;
          my $zeventsev = uc $F5;
          $df_t_perfcounter{$zcounter} = "$zwarn/$zcrit/$zoperator/$zeventsev"; 
        }

        else
        {
	  $msg = sprintf("Error - SYSTEM section parameter %s unknown - exiting - rec=%s\n",$F1,$F0);
	  &error_fatal($msg);
          exit 1;
        }

        #---- Defaults
	if ($iterations eq "*")   { $iterations = 999999; }
	if ($g_use_dns eq "")     { $g_use_dns = "Y"; }
	if ($count_tables eq "")  { $count_tables = "Y"; }
	if ($make_tickets eq "")  { $make_tickets = "N"; }
	if ($remsh_retries eq "") { $remsh_retries = 1; }
	if ($post_dir eq "")      { $post_dir = "/usr/local/bin/X11"; }
	if ($ver_colors eq "")    { $ver_colors = "N"; }
	if ($save_dir eq "")      { $save_dir = "/opt/dbamon/dat/config_save"; }
	if ($sleep_time eq "")    { $sleep_time = 30; }
	if ($opc eq "")           { $opc = "N"; }
	if ($g_pager eq "")       { $g_pager = "Y"; }
	if ($remsh_sleep eq "")   { $remsh_sleep = 5; }
	if ($df_t_sa_max_logins eq "") { $df_t_sa_max_logins = 2; }
	if ($df_t_sga_w eq "")         { $df_t_sga_w = 99; }
	if ($df_t_mssql_vlf_count eq "")        { $df_t_mssql_vlf_count = 100; }
	if ($df_t_sga_c eq "")        { $df_t_sga_c = 100; }
	if ($df_t_cf_min eq "")       { $df_t_cf_min = 1; }
	if ($df_t_db_cache_c eq "")   { $df_t_db_cache_c = "15"; }
	if ($df_t_db_cache_w eq "")   { $df_t_db_cache_c = "85"; }
	if ($df_t_nuserproc_w eq "")  { $df_t_nuserproc_w = "80"; }
	if ($df_t_nuserproc_c eq "")  { $df_t_nuserproc_c = "99"; }
	if ($df_ba_arc eq "")     { $df_ba_arc = 24*30; }
	if ($df_mssql_job_fail_check eq "")     { $df_mssql_job_fail_check = "N" ; }
	if ($df_mssql_log_min_sev    eq "")     { $df_mssql_log_min_sev    = "17" ; }
	if ($www_font eq "")              { $www_font = "arial"; }    #---- Was verdana
	if ($www_check eq "")             { $www_check = "Y"; }
	if ($www_magnify eq "")           { $www_magnify = "Y"; }

        #---- Old
	#if ($www_table_bgcolor eq "")     { $www_table_bgcolor = "cornsilk"; }
        #---- New
	if ($www_table_bgcolor eq "")     { $www_table_bgcolor = "wheat"; }

	if ($www_table_headbgcolor eq "") { $www_table_headbgcolor = "cadetblue"; }  #---- Was slateblue
	if ($debug_email eq "")     { $debug_email = $admin_email; }
	if ($g_sql_job_owner_event eq "")   { $g_sql_job_owner_event = "Y"; }
	if ($g_sql_db_owner_event  eq "")   { $g_sql_db_owner_event  = "Y"; }
	if ($g_detail_retain_days eq "")    { $g_detail_retain_days = 30; } 
	if ($g_phonehome eq "")             { $g_phonehome = "Y"; } 
	if ($g_quote_domain_users  eq "")   { $g_quote_domain_users = "N"; }

        #---- New way
        if ($config_parm{'default_mssql_tran_duration_hrs'} eq "") { $config_parm{'default_mssql_tran_duration_hrs'} = 4; } 
        if ($config_parm{'default_mssql_tran_tlog_gb'}      eq "") { $config_parm{'default_mssql_tran_tlog_gb'}      = 1; } 

      }   # End of SYSTEM section

    } # End of comment check

    if ($F1 eq "Section")
    {
      $section = $F2;
      if ($debug_read) { printf("Now Processing %s Section ...\n",$section); }
    }

  } #---- End of getline do

  close(PARMALL);

  $omsg = "";
  if ($http_details_dir ne "") { $omsg .= "Warning - dbamonrc SYSTEM section parm HTTP_Detail_Dir: is obsolete - ignored\n" }
  if ($http_rpt_path    ne "") { $omsg .= "Warning - dbamonrc SYSTEM section parm HTTP_Rpt_Path: is obsolete - ignored\n" }
  if ($report_path      ne "") { $omsg .= "Warning - dbamonrc SYSTEM section parm HTML_Rpt_Path: is obsolete - ignored\n" }
  if ($http_events_dir  ne "") { $omsg .= "Warning - dbamonrc SYSTEM section parm HTTP_Events_Dir: is obsolete - ignored\n" }
  if ($http_path        ne "") { $omsg .= "Warning - dbamonrc SYSTEM section parm HTTP_Log_Path: is obsolete - ignored\n" }
  if ($http_rpt_path    ne "") { $omsg .= "Warning - dbamonrc SYSTEM section parm HTTP_Rpt_Path: is obsolete - ignored\n" }
  if ($html_sys_path    ne "") { $omsg .= "Warning - dbamonrc SYSTEM section parm HTML_Sys_Path: is obsolete - ignored\n" }
  if ($html_events_dir  ne "") { $omsg .= "Warning - dbamonrc SYSTEM section parm HTML_Events_Dir: is obsolete - ignored\n" }
  if ($html_path        ne "") { $omsg .= "Warning - dbamonrc SYSTEM section parm HTML_Log_Path: is obsolete - ignored\n" }
  if ($images_path      ne "") { $omsg .= "Warning - dbamonrc SYSTEM section parm Images_Dir: is obsolete - ignored\n" }
  if ($use_mysql        ne "") { $omsg .= "Warning - dbamonrc SYSTEM section parm Use_MySQL: is obsolete - ignored\n" }
  if ($omsg  ne "") 
  {
    $msg = "Obsolete DBAmon dbamonrc SYSTEM Parameters Found\n $omsg";
    &error_fatal($msg);
  }

  # Assign values to html_ vars
  if ($arg eq "test") 
  {
    $html_sys_path   = "/opt/dbamon/htm/dbamon_sys_test.html";
    $html_events_dir = "/opt/dbamon/htm/events_current_test";
    $html_path       = "/opt/dbamon/htm/dbamon_log_test.html";
  }
  else
  {
    $html_sys_path   = "/opt/dbamon/htm/dbamon_sys.html";
    $html_events_dir = "/opt/dbamon/htm/events_current";
    $html_path       = "/opt/dbamon/htm/dbamon_log.html";
  } 
  # 2.03 modification - I will now write to .new path for HTML
  #   documents and will move _path to _path_perm at the end of each iteration
  $html_path_perm   = html_path;
  $html_path        = "${html_path}.new";
  
  $high_users = $i_users;

  #---- For info events with no host
  $html_events[0] = "/dev/null";
  $html_details[0] = "/dev/null";

  #---- Validate SYSTEM values
  if ($iterations eq "")    { &error_parm_null("SYSTEM","iterations") }
  if ($dbamon_admin eq "")  { &error_parm_null("SYSTEM","dbamon_admin") }
  if ($log_path eq "")      { &error_parm_null("SYSTEM","log_path") }
  if ($admin_email eq "")   { &error_parm_null("SYSTEM","admin_email") }

  if ($use_oracle ne "" && $use_oracle ne "N" && $use_oracle ne "Y") 
  {
    &error_parm("SYSTEM Section parameter Use_Oracle: must be null, Y or N");
  }
  if ($use_oracle eq "Y" && ($oracle_sid eq "" || $oracle_home eq ""))
  {
    &error_parm("SYSTEM Section: If Use_Oracle:=Y, then ORACLE_SID and ORACLE_HOME must be specified");
  }
  #if ($use_oracle ne "Y" && $www_check eq "Y")
  #{
  #  &error_parm("SYSTEM Section: If WWW_Check: is Y, then Use_Oracle: must also be Y");
  #}

  if ($use_oracle eq "Y" || $use_mssql eq "Y") 
  {
    if ($use_oracle eq "Y") { $z = "Oracle" }
    if ($use_mssql eq "Y")  { $z = "MSSQL" }
    else                    { $z = "MySql" }
    $msg = sprintf("%s Data Repository will be used",$z);
    logit($msg);
    $use_repository = "Y";
  }
  else
  {
    &error_fatal("Both Use_Oracle: and Use_MSSQL: are NULL in dbamonrc parm file: $parmfl -- You MUST use a repository");
  }

  #---- Wrapup

  $msg = sprintf("Read %s userlist definitions",$high_users);
  logit($msg);

  #---- Old
  ## Read WWW_Check:
  #chop($font = `grep 'WWW_Font:' $parmfl | grep -v '^#' | awk '{ print \$2 }'`);
  #if ($font =~ /%/)
  #{
  #  $font =~ s/%/ /g;
  #  $font   = "'$font'";
  #  $fontdq = "\"$font\"";
  #}

  #---- New
  $font   = $www_font;
  $fontdq = $www_font;

  #---- A few hard-coded parms
  $df_redo_rate     = 12;
  $df_redo_fs_ratio = 1.1;

  #---- Debugging
  if ($debug_read) 
  {
    foreach (sort keys %config_parm)
    {
      print "config_parm{$_}=$config_parm{$_}\n"; 
    }
  }

  &debug(-1,"DEBUG - id5_label=$id5_label df_ba_arc=$df_ba_arc");
}

# -----------------------------------------------------------------------------
1;
