sub remsh
#--------------------------------------------------------------------------------
#
# remsh.pl
#
#--------------------------------------------------------------------------------
# Changes:
#  bb 01Jun04: Change to logic for retrying "AccountLocked" errors. We now want
#   to retry these because they after 5 minutes will automatically unlock. Also
#   changed remsh_sleep from 30 to 90.
#  bb 04Jun04: Moved the setting of remsh_sleep to dbamonrc.
#  bb 20Oct04: Added xx to DBA293 event creation logic: && $remsh_try >= $remsh_retries
#  bb 02Nov04: For 'login incorrect' I want to try letting that retry. 
#  bb 16Dec04: Changed sev of DBA293 from C to W. 
#  bb 10Feb04: Added "trust" network problem 
#  bb 07Oct07: New logic to retry after MSSQL7 bug 
#  bb 14Mar11: New logic for altering the ping command bases on thisos 
#  bb 26Sep11: "my" for @file_array and $file_contents
#  bb 04Jun12: New remsherr_short logic
#--------------------------------------------------------------------------------
{
  my ($c, $file, $h) = @_;

  my @file_array;
  my $file_contents;
  my $file_contents_l = 0; 
  my $file_array_l = 0; 
  my $remsherr       = "";
  my $remsherr_short = "";

  my $ev_start = time; 
  my $ev_elapsed = 0; 

  ##print "DEBUG1001 - remsh - c=$c\n"; 

  #---- Build command including prefix and suffix
  $cmdall = $cmdpfx[$h] . $c . $cmdsfx[$h];
  &debug($h, "cmdpfx=$cmdpfx[$h] - cmdall=$cmdall"); 

  #---- Debug
  ##print "cmdpfx[$h]=$cmdpfx[$h]\n";
  ##print "c=$c\n";
  ##print "cmdsfx[$h]=$cmdsfx[$h]\n";

  #---- Clear, delete and init a logfile
  open(TMPFL7,">$tmpfl7") ||  &error_msg("Can't open output $tmpfl7 - $!");
  print TMPFL7 "Command Input:\n";
  print TMPFL7 "$cmdall \n";

  #---- Clear and delete remshlog
  close(REMSHLOG);
  unlink $remshlog; 

  #---- Del output file (2012/07/16)
  unlink $file; 

  $remsh_try = 0;
  $remsh_stop = 0;
  $conshort = "Undiagnosed";
  $file_contents = ""; 

  while ($remsh_try++ <= $remsh_retries)
  {
    #---- It may be that someone inhibited it while it was
    #---- in retry logic, so check every cycle.
    #---- If we are inhibited, then don't bother
    if (&check_inhib($h) eq "Y")
    {
      $up[$h]  = "I";
      $rc_save = -2;
      return;
    }

    #---- Issue command

    if ($probe_timeout > 0)
    {
      $start_remsh_time = time;

      ##$SIG{CHLD} = \&REAPER;   # TEST

      eval {
	alarm ($probe_timeout);
        $rc = system("$cmdall > $file 2> $remshlog");
	alarm (0);
      };

      ##$SIG{CHLD} = 'DEFAULT';   # TEST

      alarm(0);    # Just in case... turn off alarm
      $elapsed_remsh_time = time - $start_remsh_time;

      ##print "DEBUG - $titleraw[$h] - probe_timeout=$probe_timeout - elapsed_remsh_time=$elapsed_remsh_time\n" if $debug_293 eq "Y";

      if ($@)      #---- Did eval fail?
      {
	chop($eval_msg = $@);
	print "DBAmon Internal - remsh Exception: ($eval_msg)\n";
        if ($eval_msg =~ /TIMEOUT/ && $remsh_try >= $remsh_retries)
        {
	  $up[$h] = "N";
	  $short = "ProbeTimeout";
	  open(TMPFL7,">>$tmpfl7") ||  &error_msg("Can't open output $tmpfl7 - $!");
	  print  TMPFL7 "\nThe remsh/ssh connect to $hostname[$h] ran for $elapsed_remsh_time seconds which is >= than the\n";
	  print  TMPFL7 "(dbamonrc) Probe_Timeout: value of $probe_timeout, and the command was cancelled. \n";
	  print  TMPFL7 "This may be an indication that this DB instance or server is hung.\n";
	  print  TMPFL7 "Log File(s): \n";
	  print  TMPFL7 `cat $remshlog`;
	  print  TMPFL7 `cat $file`;
	  close(TMPFL7);
          &event("W",$thishost,"Probe Connect TIMEOUT (dbamonrc:Probe_Timeout=$probe_timeout)","ProbeTimeout","DBA293",$tmpfl7);
          system("cat $tmpfl7 | dbamon_mailer -m -s 'DBAmon Remsh TIMEOUT: $titleraw[$h] remsh_try=$remsh_try remsh_retries=$remsh_retries' $admin_email") if $debug_293 eq "Y";
        }
      }
    }
    else
    {
      #---- Actual remsh command is run here
      &logit("(remsh) Running cmdall=($cmdall)",$h);
      $rc = system("$cmdall > $file 2> $remshlog");
    }

    if (
        $comm_method[$thishost] eq "ssh" &&
        $dbms[$thishost] eq "MSSQL" &&
        $rc == 1280
       )
    {
      &logit("Changing bogus ssh rc=5 to rc=0",$h);
      $rc = 0;
    }

    #---- Old way vs. new way
    if (1 == 0)
    {
      @file_array      = `cat $file`;
      $file_contents   = `cat $file`;
      #---- Capture possible error in remsherr var
      chop($remsherr = `cat $remshlog | grep -v 'Warning: Remote host denied X11 forwarding' | grep -v 'otrace/admin'`); 
    }
    else
    {
      @file_array      = &fileReadArray($file);
      $file_contents   = &fileReadScalar($file);
      $remsherr        = &fileReadScalar($remshlog);
    }

    #---- populate remsherr_short
    $remsherr_short = $remsherr;
    if (length($remsherr_short) > 180)
    {
      $remsherr_short = substr($remsherr_short,0,180); 
    }

    #---- Some cleanup of remsherr_short
    $remsherr_short =~ s/HASH\(.*\)/HASH(zzz)/;
    $remsherr_short =~ s/\n//g; 
    $remsherr_short =~ s/\'//g; 

    $file_contents_l = length($file_contents);
    $file_array_l    = $#file_array;

    &debug($h,"AFTER REMSH - titleraw[$h]=${titleraw[$h]} file=$file c=$c rc=$rc");
    &debug($h,"AFTER REMSH -   file_contents_l=$file_contents_l file_array_l=$file_array_l remsherr=$remsherr");

    #---- PHONEHOME if so requested by PROBE
    if ($file_contents =~ /PHONEHOME:/ && $g_phonehome eq "Y")
    {
      $zmsg_text = ""; 
      $zmsg_cnt  = 0; 
      #---- Find PHONEHOME detail line
      open(PI, "$file") || &error_msg("Cannot open input file=$file - $!"); 
      while(<PI>)
      {
        if (/PHONEHOME:/)
        { 
          @zwords = split; 
          $zmsg_text .= $zwords[$#zwords] . " "; 
          $zmsg_cnt++; 
        } 
      }
      close(PI); 
      system("cat $file | dbamon_mailer -s 'DBAmon | $titleraw[$h] | Probe PHONEHOME: ($zmsg_text/$zmsg_cnt)' $admin_email");
      &logit("... Probe output contains PHONEHOME",$h);
    }

    #---- BB 15Apr04 Test
    if ($file_contents_l < 10 && 
        $c !~ /^erase / &&
        $c !~ /^rm / &&
        $c !~ /^cksum / &&
        $rc == 0)
    {
      ##system("cat $file | dbamon_mailer -m -s 'DBAmon Remsh - $titleraw[$h] - FILE_CONTENTS_SMALL_LT_10 - l=$file_contents_l' $admin_email");
      &logit("DEBUG: Changing rc $rc/-2 - titleraw[$h]=${titleraw[$h]} c=$c file_contents_l=$file_contents_l");
      &logit("DEBUG:   file_contents=$file_contents");
      $rc = -2;
    }

    $rcdebug = 0;
    if ($rcdebug)  
    {
      &logit("DEBUG: h=$h cmdall=$cmdall rc=$rc ");
      &logit("DEBUG: cmdpfx=$cmdpfx[$h] c=$c cmdsfx=$cmdsfx[$h] ");
      &logit("DEBUG: remshlog contents=" . `cat $remshlog`);
      &logit("DEBUG: file $file contents=" . `cat $file`);
    }
   
    my $pingflag = "";
    my $pingable = "";


    ##print "remsherr=$remsherr\n"; 

    # Look for possible NT error
    #perf# if (system("grep  'AUTO-INHIBIT' $file") == 0) 
    if ( scalar(grep(/AUTO-INHIBIT/, @file_array)) > 0) 
    {
      &event("I",$thishost,"DBAmon checking inhibited by AUTO-INHIBIT","InhibAuto","DBA304I","$file");
      $up[$h] = "I";
      return;
    }

    #---- See if somehow the req file disappeared
    if ($file_contents =~ /REQFILEMISSING/) 
    {
      &logit("REQFILEMISSING - Require file disappeared - turning off download flag",$h);
      &downloadFlagOff($h);
    }

    # Look for truncated probe output - will retry below
    #perf# if (
    #perf#     system("grep 'Starting' $file > /dev/null ") == 0 && 
    #perf#     system("grep 'dbamon' $file > /dev/null ") == 0 && 
    #perf#     system("grep 'END_OF_PROBE' $file > /dev/null ") != 0
    #perf#    ) 
    if (
           scalar(grep(/Starting/, @file_array)) > 0
        && scalar(grep(/dbamon/, @file_array)) > 0 
        && scalar(grep(/END_OF_PROBE/, @file_array)) == 0 
       ) 
    {
      $rc = -1;
      open(REMSHLOG,">$remshlog") ||  &error_msg("Can't open output $remshlog - $!");
      print REMSHLOG "END_OF_PROBE Not Found in probe output \n";
      close(REMSHLOG);
    }

    # Look for possible NT error
    #perf# if (system("grep  'Access Denied' $file") == 0) 
    if (scalar(grep(/Access Denied/, @file_array)) > 0) 
    {
      $rc = -1;
      open(REMSHLOG,">$remshlog") ||  &error_msg("Can't open output $remshlog - $!");
      print REMSHLOG "Access Denied \n";
      close(REMSHLOG);
    }

    # remsh worked - look for downloaded pgm's disappearing
    ## OLD TEST if ($remsherr =~ /not found/ || $remsherr =~ /No such file/)
    ## OLD2 ($remsherr =~ /not found/ && $remsherr =~ /ksh: \/tmp\/$probe_dir[$h]\/dbamon_/) ||
    if (
        ($remsherr =~ /not found/ && $remsherr =~ /ksh: \/tmp\/dbamon_/) ||
        ($remsherr =~ /not found/ && $remsherr =~ /ksh: $probe_dir[$h]\/dbamon_/) ||
        $remsherr =~ /open perl script/          
       )
    {
      print "dlfoff-1 $titleraw[$h] remsherr=$remsherr \n rc=$rc\n";
      print "dlfoff-1 rc=$rc\n";
      &downloadFlagOff($h);
      open(TMPFL7,">>$tmpfl7") ||  &error_msg("Can't open output $tmpfl7 - $!");
      print  TMPFL7 "Log File(s): \n";
      print  TMPFL7 `cat $remshlog`;
      print  TMPFL7 `cat $file`;
      close(TMPFL7);
      &event("W",$thishost,"Downloaded DBAmon Probe Software Not Found - Will retry download on next iteration","ProbeGone","DBA291",$tmpfl7);
      $up[$h] = "D";
      return;
    }

    # Look for compilation error
    if ($remsherr =~ /compilation error/)
    {
      &event("C",$thishost,"Connect resulted in Probe compilation error","Compile","DBA292",$remshlog);
    }

    #---- Look for DBD SQL Errors 
    if ($remsherr =~ /DBD::ODBC::/)
    {
      &event("C",$thishost,"Connect resulted in Probe DBD SQL Error(s)","Probe_SQL_Errors","DBA292",$remshlog);
    }

    if ($rc == 0)
    {
      ##if ($remsh_try > 1) 
      ##{
      ##  system("echo ' ' | dbamon_mailer -m -s 'DBAmon Remsh Retry Worked For: $title[$h] $remsh_try' $admin_email");
      ##}
      $must_complete = 0;

      if ($remsherr ne "")
      {
	#---- Test - append remshlog to output file
	open(APPEND,">>$file") ||  &error_msg("Can't open output $file - $!");
        print APPEND "\n---------- remsh Diagnostic Output: ----------\n";
	printf APPEND "Command: $cmdall\n";
	printf APPEND $remsherr;
	close(APPEND);
      }

      #---- Look for errors that have slipped by
      ## $file_contents = `cat $file` if $file_contents eq "";

      &error_msg("remsh L274 - file_contests Should not be null") if $file_contents eq "";

      if ($routine !~ /Download/ && $file_contents =~ /dbamon_mssql/ && $file_contents =~ /dbamon_ora/)
      {
	system("cat $file | dbamon_mailer -s 'DBAmon remsh() MSSQL&ORA $titleraw[$h] Corrupt' $debug_email");
	$msg = sprintf("... Probe output corrupted (MSSQL & ORA) - Will retry");
	&logit($msg,$h);
        chop($datefile = `date +%C%y%m%d_%H%M%S`);
        system("cp $file /tmp/dbamon_debug_corrupt_${titleraw[$h]}_${datefile}.txt"); 
        next; 
      }
      if ($routine !~ /Download/ 
          && $file_contents =~ /Describe failed during DBI/ 
          && $file_contents =~ /FETCH\{odbc_more_results\}/
         )
      {
	system("cat $file | dbamon_mailer -s 'DBAmon remsh() MSSQL7-Describe Problem $titleraw[$h] Corrupt' $debug_email");
	$msg = sprintf("... MSSQL7 Describe Bug - Will retry");
	&logit($msg,$h);
        chop($datefile = `date +%C%y%m%d_%H%M%S`);
        system("cp $file /tmp/dbamon_debug_corrupt_${titleraw[$h]}_${datefile}.txt"); 
        next; 
      }

      $ev_elapsed = time - $ev_start;
      &debug($h,"END of REMSH 1 - ev_elapsed=$ev_elapsed rc_save=$rc_save");

      return;
    }

    ##print "remshlog="
    ##system("cat " remshlog)

    #---- Diagnose possible errors

    if ($remsherr =~ /Connection refused/) 
    {
      $conshort = "ConnRefused";
    }
    if ($remsherr =~ /Connection reset/i) 
    {
      $conshort = "ConnReset";
    }

    if ($remsherr =~ /Connection closed/i) 
    {
      $conshort = "ConnClosed";
    }

    if ($remsherr =~ /account is currently locked/)
    {
      $conshort = "AccountLocked";
      #system("cat $file     | dbamon_mailer -s 'DBAmon Remsh AccountLocked: $titleraw[$h] - file    ' $admin_email ");
      #system("cat $remshlog | dbamon_mailer -s 'DBAmon Remsh AccountLocked: $titleraw[$h] - remshlog' $admin_email ");
      #---- BB 01Jun04 - Comment this line so that it retries
      #$remsh_stop = 1;
    }
    if ($remsherr =~ /Account is disabled/) 
    {
      $conshort = "AccountDisabled";
      $remsh_stop = 1;
    }
    if ($remsherr =~ /The workstation does not have a trust secret/) 
    {
      $conshort = "WindowsTrust";
      $remsh_stop = 1;
    }
    if ($remsherr =~ /Password for this account expired/) 
    {
      $conshort = "PWExpired";
      $remsh_stop = 1;
    }
    if ($remsherr =~ /Password Expired/) 
    {
      $conshort = "PWExpired";
      $remsh_stop = 1;
    }
    if ($remsherr =~ /Access Denied/) 
    {
      $conshort = "AccessDenied";
    }
    if ($remsherr =~ /Connection timed out/) 
    {
      $conshort = "NetTimeout";
    }
    if ($remsherr =~ /Login incorrect/) 
    {
      $conshort = "LoginIncorrect";

      #---- Test change - for this error, do let it retry - commenting this next line
      #$remsh_stop = 1;

      print "dlfoff-2 $titleraw[$h]\n";
      &downloadFlagOff($h);
    }
    
    #---- I will leave this here for now - it's handy
    print "remsh() Error: $titleraw[$h] | rc=$rc remsh_stop=$remsh_stop remsherr=$remsherr conshort=$conshort\n";

    if (
        ($remsherr =~ /not found/ && $remsherr =~ /ksh: \/tmp\/dbamon_/) ||
        $remsherr =~ /open perl script/          
       )
    {
      $conshort = "SWDisappeared";
      $remsh_stop = 1;
      &downloadFlagOff($h);
      $rc = -1;    # Test
      ## print "dlfoff-3 $titleraw[$h] rc=$rc remsh_stop=$remsh_stop\n";
    }

    # See if END_OF_PROBE is present. If it isn't, then retry
    #perf# if (
    #perf#     system("grep 'Starting' $file > /dev/null ") == 0 && 
    #perf#     system("grep 'dbamon' $file > /dev/null ") == 0 && 
    #perf#     system("grep 'END_OF_PROBE' $file > /dev/null ") != 0 && $remsh_stop != 1
    #perf#    )
    if (
            scalar(grep(/Starting/, @file_array)) > 0
         && scalar(grep(/dbamon/, @file_array)) > 0
         && scalar(grep(/END_OF_PROBE/, @file_array)) == 0
       ) 
    {
      $conshort = "END_OF_PROBE_N/F";
      system("cat $file | dbamon_mailer -s 'DBAmon remsh() END_OF_PROBE Not Found $titleraw[$h] remsh_try=$remsh_try remsherr=$remsherr' $admin_email") if $remsh_try > 3;
      if ($remsh_try <= $remsh_retries)
      {
	$msg = sprintf("... Remsh Connect Attempt:%s Failed - END_OF_PROBE NF - Err=[%s] RC=%i - Will Retry - Sleeping %s ...",$remsh_try,$remsherr_short,$rc,$remsh_sleep);
	&logit($msg,$h);
	#perf# system("sleep $remsh_sleep");
	sleep $remsh_sleep;
	next;
      }
    }

    # It error'd on < the last attempt
    if ($rc != 0 && $remsh_try <= $remsh_retries && $remsh_stop != 1)
    {
      # Log message
      $msg = sprintf("... Remsh Connect Attempt:%s Failed Err=[%s] RC=%i - Will Retry - Sleeping %s ...",$remsh_try,$remsherr_short,$rc,$remsh_sleep);
      &logit($msg,$h);
      #perf# system("sleep $remsh_sleep");
      sleep $remsh_sleep;
      next;
    }

    # It error'd on what will be the last attempt
    if ($rc != 0 && ($remsh_try > $remsh_retries || $remsh_stop))
    {
      # See if server is pingable
      
      #---- 2011/03/14 BB - New logic for cygwin - it is using the WIN ping command with different syntax
      if ($thisos =~ /cygwin/i) { $pingcmd = "ping $hostname[$h] -l 64 -n 2"; } 
      else                      { $pingcmd = "ping $hostname[$h] 64 2"; } 

      ##$rc = system("ping $hostname[$h] 64 2 > $pinglog 2>&1");
      $rc = system("$pingcmd > $pinglog 2>&1");
      if ($rc != 0 || system("grep -i '100% packet loss' $pinglog") == 0)
      {
	$pingable = "Is *NOT* Ping-able";
        $pingflag = "N";
        print "dlfoff-4 $titleraw[$h]\n";
        &downloadFlagOff($h);
      }
      else
      {
	$pingable = "Is Ping-able";
        $pingflag = "Y";
      }

      # Create report
      open(TMPFL7,">>$tmpfl7") ||  &error_msg("Can't open output $tmpfl7 - $!");
      printf TMPFL7 "Remsh Attempt: %s RC=%s - Error Details:\n",$remsh_try,$rc;
      print  TMPFL7 `cat $remshlog`;
      print  TMPFL7 `cat $file`;
      print  TMPFL7 "\nHost: $hostname[$h] \n";
      print  TMPFL7 "remsherr_short: $remsherr_short \n";
      print  TMPFL7 "remsherr: $remsherr \n";
      close(TMPFL7);

      # Log message
      if ($remsh_sev eq "C" || $must_connect)
      {
        close(TMPFL7);
        $ev_cat = "Avail";
        &event("C",$thishost,"Connect Failure: $conshort - After $remsh_try Attempt(s) - Server $pingable - ErrShrt=($remsherr_short) RC=$rc","Connect","DBA290",$tmpfl7);
      }
      else
      {
        close(TMPFL7);
        $ev_cat = "Avail";
        &event("W",$thishost,"Connect Failure: $conshort - After $remsh_try Attempt(s) - Server $pingable - ErrShrt=($remsherr_short) RC=$rc","Connect","DBA290",$tmpfl7);
      }

      $up[$h] = "N";
      $short = "Rsh";
      last;
    }
  } # End of retries while loop

  #---- Look for errors that have slipped by

  $file_contents = `cat $file` if $file_contents eq "";
  if ($file_contents =~ /dbamon_mssql/ && $file_contents =~ /dbamon_ora/)
  {
    system("cat $file | dbamon_email -s 'DBAmon remsh() MSSQL & ORA $titleraw[$h] remsherr=$remsherr' $admin_email");
    $msg = sprintf("... Probe output corrupted (MSSQL & ORA) 2");
    &logit($msg,$h);
  }

  ## # See if END_OF_PROBE is present. If it isn't, create event
  ## if ($up eq "" && system("grep 'END_OF_PROBE' $file") != 0)
  ## {
  ##   $msg = sprintf("... Probe END_OF_PROBE Not Found - Will NOT Retry");
  ##   &logit($msg,$h);
  ##   system("cat $file | dbamon_mailer -s 'DBAmon remsh() END_OF_PROBE Not Found $titleraw[$h] Will NOT Retry remsherr=$remsherr' $admin_email");
  ## }

  #---- Set vars
  $rc_save = $rc;
  $must_connect = 0;
 
  $ev_elapsed = time - $ev_start;
  &debug($h,"END of REMSH 2 - ev_elapsed=$ev_elapsed rc_save=$rc_save");

  return;
}

#-------------------------------------------------------------------------------
1;
